<?php defined('ABSPATH') OR exit;
/*
Plugin Name: Gift upon purchase for WooCommerce
Description: This plugin will help create a gift when buying for WooCommerce
Tags: gift, product, woocommerce, bonus
Author: Maxim Glazunov
Author URI: https://icopydoc.ru
License: GPLv2
Version: 1.2.0
Text Domain: gift-upon-purchase-for-woocommerce
Domain Path: /languages/
WC requires at least: 3.2.0
WC tested up to: 5.1.0
*/
/*	Copyright YEAR PLUGIN_AUTHOR_NAME (email : djdiplomat@yandex.ru)
 
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License, version 2, as
	published by the Free Software Foundation.
 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/
require_once plugin_dir_path(__FILE__).'/functions.php'; // Подключаем файл функций
register_activation_hook(__FILE__, array('GiftUponPurchaseForWooCommerce', 'on_activation'));
register_deactivation_hook(__FILE__, array('GiftUponPurchaseForWooCommerce', 'on_deactivation'));
register_uninstall_hook(__FILE__, array('GiftUponPurchaseForWooCommerce', 'on_uninstall'));
add_action('plugins_loaded', array('GiftUponPurchaseForWooCommerce', 'init'));
add_action('plugins_loaded', 'gupfw_load_plugin_textdomain'); // load translation
function gupfw_load_plugin_textdomain() {
 load_plugin_textdomain('gupfw', false, dirname(plugin_basename(__FILE__)).'/languages/');
}
class GiftUponPurchaseForWooCommerce {
 protected static $instance;
 public static function init() {
	is_null(self::$instance) AND self::$instance = new self;
	return self::$instance;
 }

 public function __construct() {
	// gupfw_DIR contains /home/p135/www/site.ru/wp-content/plugins/myplagin/
	define('gupfw_DIR', plugin_dir_path(__FILE__)); 
	// gupfw_URL contains http://site.ru/wp-content/plugins/myplagin/
	define('gupfw_URL', plugin_dir_url(__FILE__));
	// gupfw_UPLOAD_DIR contains /home/p256/www/site.ru/wp-content/uploads
	$upload_dir = (object)wp_get_upload_dir();
	define('gupfw_UPLOAD_DIR', $upload_dir->basedir);
	// gupfw_UPLOAD_DIR contains /home/p256/www/site.ru/wp-content/uploads/gift-upon-purchase-for-woocommerce
	$name_dir = $upload_dir->basedir."/gift-upon-purchase-for-woocommerce"; 
	define('gupfw_NAME_DIR', $name_dir);
	$gupfw_keeplogs = gupfw_optionGET('gupfw_keeplogs');
	define('gupfw_KEEPLOGS', $gupfw_keeplogs);
	define('gupfw_VER', '1.2.0');
	$gupfw_version = gupfw_optionGET('gupfw_version');
  	if ($gupfw_version !== gupfw_VER) {gupfw_set_new_options();} // автообновим настройки, если нужно	

	add_action('admin_menu', array($this, 'add_admin_menu'));
	add_action('admin_notices', array($this, 'gupfw_admin_notices_function'));
	add_filter('plugin_action_links', array($this, 'gupfw_plugin_action_links'), 10, 2);

	// https://wpruse.ru/woocommerce/custom-fields-in-products/
	// https://wpruse.ru/woocommerce/custom-fields-in-variations/
	add_filter('woocommerce_product_data_tabs', array($this, 'gupfw_added_wc_tabs'), 10, 1);
	add_action('admin_footer', array($this, 'gupfw_art_added_tabs_icon'), 10, 1);
	add_action('woocommerce_product_data_panels', array($this, 'gupfw_art_added_tabs_panel'), 10, 1);
	add_action('woocommerce_process_product_meta',  array($this, 'gupfw_art_woo_custom_fields_save'), 10, 1);

	// https://github.com/woocommerce/selectWoo
	// https://rudrastyh.com/wordpress/select2-for-metaboxes-with-ajax.html	
	add_action('wp_ajax_gupfwselect2', array($this, 'gupfw_get_posts_ajax_callback'), 10, 1); // wp_ajax_{action}	

	// https://www.kobzarev.com/wordpress/woo-visual-hook/
	// https://businessbloomer.com/woocommerce-visual-hook-guide-single-product-page/ 
	add_action('woocommerce_product_meta_start', array($this, 'gupfw_art_get_text_field_after_add_card'), 10, 1);

	// https://www.pandoge.com/haki/skidka-v-procentah-na-vsyu-korzinu-bez-kupona-v-woocommerce
	add_action('template_redirect', array($this, 'gupfw_add_product_to_cart'), 10, 1);
	// https://rudrastyh.com/woocommerce/change-product-prices-in-cart.html
	add_action('woocommerce_before_calculate_totals', array($this, 'gupfw_change_recalc_price'), 10, 1);

	add_filter('woocommerce_cart_item_name', array($this, 'gupfw_add_excerpt_in_cart_item_name'), 10, 4);

	// отказ от подарков
	add_action('set_current_user', array($this, 'gupfw_set_new_cookie'), 10, 1);
	add_action('woocommerce_cart_coupon', array($this, 'gupfw_before_add_to_cart_btn'), 10, 1);

	/* Регаем стили только для страницы настроек плагина */
	add_action('admin_init', function() {
		wp_register_style('gupfw-admin-css', plugins_url('css/gupfw.css', __FILE__));
	}, 9999);
 }

 // Срабатывает при активации плагина (вызывается единожды)
 public static function on_activation() {
	$upload_dir = (object)wp_get_upload_dir();
	$name_dir = $upload_dir->basedir.'/gift-upon-purchase-for-woocommerce';
	if (!is_dir($name_dir)) {
	 if (!mkdir($name_dir)) {
		error_log('ERROR: Ошибка создания папки '.$name_dir.'; Файл: gift-upon-purchase-for-woocommerce.php; Строка: '.__LINE__, 0);
	 }
	}

	if (is_multisite()) {
		add_blog_option(get_current_blog_id(), 'gupfw_version', '1.2.0');
		add_blog_option(get_current_blog_id(), 'gupfw_keeplogs', '0');
		add_blog_option(get_current_blog_id(), 'gupfw_disable_notices', '0');	
		add_blog_option(get_current_blog_id(), 'gupfw_errors', '');
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_status', 'show');
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_content', 'gift for purchase'); 
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_color', '#000000');
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_fsize', '12');
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_remove_gift_in_cart', __('Remove gifts', 'gupfw'));
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_accept_gift_in_cart', __('Accept gifts', 'gupfw'));
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_status', 'show');
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_content', 'gift for purchase'); 
		add_blog_option(get_current_blog_id(), 'gupfw_gift_for_any_product_in_cart_content', 'gift');
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_color', '#000000');
		add_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_fsize', '12');
		add_blog_option(get_current_blog_id(), 'gupfw_gift_for_any_product_arr', array());
		add_blog_option(get_current_blog_id(), 'gupfw_cart_total_price', '0');
		add_blog_option(get_current_blog_id(), 'gupfw_whose_price_exceeds', '0');
		add_blog_option(get_current_blog_id(), 'gupfw_days_of_the_week', array());
		add_blog_option(get_current_blog_id(), 'gupfw_days_of_the_hours', array());	
	} else {
		add_option('gupfw_version', '1.2.0', '', 'no');
		add_option('gupfw_keeplogs', '0', '');
		add_option('gupfw_disable_notices', '0', '', 'no');
		add_option('gupfw_errors', '', '', 'no');
		add_option('gupfw_tgfp_in_cart_status', 'show');
		add_option('gupfw_tgfp_in_cart_content', 'gift for purchase'); 
		add_option('gupfw_tgfp_in_cart_color', '#000000');
		add_option('gupfw_tgfp_in_cart_fsize', '12');
		add_option('gupfw_tgfp_remove_gift_in_cart', __('Remove gifts', 'gupfw')); 
		add_option('gupfw_tgfp_accept_gift_in_cart', __('Accept gifts', 'gupfw'));
		add_option('gupfw_tgfp_in_product_status', 'show');
		add_option('gupfw_tgfp_in_product_content', 'gift for purchase'); 
		add_option('gupfw_gift_for_any_product_in_cart_content', 'gift');
		add_option('gupfw_tgfp_in_product_color', '#000000');
		add_option('gupfw_tgfp_in_product_fsize', '12');
		add_option('gupfw_gift_for_any_product_arr', array());
		add_option('gupfw_cart_total_price', '0');
		add_option('gupfw_whose_price_exceeds', '0');
		add_option('gupfw_days_of_the_week', array());
		add_option('gupfw_days_of_the_hours', array());		
	}	
 }
 
 // Срабатывает при отключении плагина (вызывается единожды)
 public static function on_deactivation() {

 } 
 
 // Срабатывает при удалении плагина (вызывается единожды)
 public static function on_uninstall() {
	if (is_multisite()) {		
		delete_blog_option(get_current_blog_id(), 'gupfw_keeplogs');
		delete_blog_option(get_current_blog_id(), 'gupfw_disable_notices');
		delete_blog_option(get_current_blog_id(), 'gupfw_errors');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_status');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_content'); 
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_color');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_cart_fsize');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_remove_gift_in_cart'); 
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_accept_gift_in_cart');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_status');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_content');
		delete_blog_option(get_current_blog_id(), 'gupfw_gift_for_any_product_in_cart_content');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_color');
		delete_blog_option(get_current_blog_id(), 'gupfw_tgfp_in_product_fsize');
		delete_blog_option(get_current_blog_id(), 'gupfw_gift_for_any_product_arr');
		delete_blog_option(get_current_blog_id(), 'gupfw_cart_total_price');
		delete_blog_option(get_current_blog_id(), 'gupfw_whose_price_exceeds');
		delete_blog_option(get_current_blog_id(), 'gupfw_days_of_the_week');
		delete_blog_option(get_current_blog_id(), 'gupfw_days_of_the_hours');
	} else {
		delete_option('gupfw_keeplogs');
		delete_option('gupfw_disable_notices');	
		delete_option('gupfw_errors');
		delete_option('gupfw_tgfp_in_cart_status');
		delete_option('gupfw_tgfp_in_cart_content');
		delete_option('gupfw_tgfp_in_cart_color');
		delete_option('gupfw_tgfp_in_cart_fsize');
		delete_option('gupfw_tgfp_remove_gift_in_cart');
		delete_option('gupfw_tgfp_accept_gift_in_cart');
		delete_option('gupfw_tgfp_in_product_status');
		delete_option('gupfw_tgfp_in_product_content');
		delete_option('gupfw_gift_for_any_product_in_cart_content');
		delete_option('gupfw_tgfp_in_product_color');
		delete_option('gupfw_tgfp_in_product_fsize');
		delete_option('gupfw_gift_for_any_product_arr');
		delete_option('gupfw_cart_total_price');
		delete_option('gupfw_whose_price_exceeds');
		delete_option('gupfw_days_of_the_week');
		delete_option('gupfw_days_of_the_hours');		
	}
 }

 // Добавляем пункты меню
 public function add_admin_menu() {
	$page_suffix = add_menu_page(null , __('Settings Gift', 'gupfw'), 'manage_options', 'gupfwsettings', 'gupfw_settings_page', 'dashicons-buddicons-groups', 51);
	require_once gupfw_DIR.'/settings.php'; // Подключаем файл настроек
	// создаём хук, чтобы стили выводились только на странице настроек
	add_action('admin_print_styles-'. $page_suffix, array($this, 'gupfw_admin_css_func'));
 	add_action('admin_print_styles-'. $page_suffix, array($this, 'gupfw_admin_head_css_func'));

	add_submenu_page('gupfwsettings', __('Debug', 'gupfw'), __('Debug page', 'gupfw'), 'manage_options', 'gupfwdebug', 'gupfw_debug_page');
	require_once gupfw_DIR.'/debug.php';
 }

 public function gupfw_admin_css_func() {
	/* Ставим css-файл в очередь на вывод */
	wp_enqueue_style('gupfw-admin-css');
	wp_enqueue_style('wp-color-picker');
	wp_enqueue_script('wp-color-picker');
	add_action('admin_footer', array($this, 'gupfw_admin_footer_script'), 99 );

	wp_enqueue_style('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/css/select2.min.css' );
	wp_enqueue_script('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/js/select2.min.js', array('jquery'));
 
	// please create also an empty JS file in your theme directory and include it too
	wp_enqueue_script('wplspms_orders', plugin_dir_url(__FILE__) . 'js/select2.js', array('jquery', 'select2'));
 } 

 public static function gupfw_admin_head_css_func() {
	/* печатаем css в шапке админки */
	print '<style>/* Gift upon purchase for WooCommerce */
		.metabox-holder .postbox-container .empty-container {height: auto !important;}
		.icp_img1 {background-image: url('. gupfw_URL .'/img/sl1.jpg);}
		.icp_img2 {background-image: url('. gupfw_URL .'/img/sl2.jpg);}
		.icp_img3 {background-image: url('. gupfw_URL .'/img/sl3.jpg);}
		.icp_img4 {background-image: url('. gupfw_URL .'/img/sl4.jpg);}
		.icp_img5 {background-image: url('. gupfw_URL .'/img/sl5.jpg);}
		.icp_img6 {background-image: url('. gupfw_URL .'/img/sl6.jpg);}
		.icp_img7 {background-image: url('. gupfw_URL .'/img/sl7.jpg);}
	</style>';
 }   

 // Подключаем свой скрпит в подвал 
 public function gupfw_admin_footer_script() { 
	// https://wp-kama.ru/id_4621/vyibora-tsveta-iris-color-picker-v-wordpress.html 
	// http://automattic.github.io/Iris/
	?>
	<script type="text/javascript">jQuery(document).ready(function($) { 
		var myOptions = {
			// устанавливает цвет по умолчанию, также цвет по умолчанию из атрибута value у input
			defaultColor: false,
			// функция обратного вызова, срабатывающая каждый раз при выборе цвета (когда водите мышкой по палитре)
			change: function(event, ui){ },
			// функция обратного вызова, срабатывающая при очистке (сбросе) цвета
			clear: function(){ },
			// спрятать ли выбор цвета при загрузке палитра будет появляться при клике
			hide: true,
			// показывать ли группу стандартных цветов внизу палитры 
			// можно добавить свои цвета указав их в массиве: ['#125', '#459', '#78b', '#ab0', '#de3', '#f0f']
			palettes: true
		}
		$('#gupfw_tgfp_in_cart_color').wpColorPicker(myOptions);
		$('#gupfw_tgfp_in_product_color').wpColorPicker(myOptions);
	});</script><?php
 } 

 // Вывод различных notices
 public function gupfw_admin_notices_function() {
	if (gupfw_optionGET('gupfw_magazin_type') === 'woocommerce') { 
		if (!class_exists('WooCommerce')) {
			print '<div class="notice error is-dismissible"><p>'. __('WooCommerce is not active!', 'gupfw'). '.</p></div>';
		}
	}

	if (isset($_REQUEST['gupfw_submit_action_settings'])) {
		if (!empty($_POST) && check_admin_referer('gupfw_nonce_action_settings', 'gupfw_nonce_field_settings')) {
			if (isset($_POST['gupfw_tgfp_in_cart_status'])) {
				gupfw_optionUPD('gupfw_tgfp_in_cart_status', sanitize_text_field($_POST['gupfw_tgfp_in_cart_status']));
			}
			if (isset($_POST['gupfw_tgfp_in_cart_content'])) {
				gupfw_optionUPD('gupfw_tgfp_in_cart_content', sanitize_text_field($_POST['gupfw_tgfp_in_cart_content']));
			}
			if (isset($_POST['gupfw_tgfp_in_cart_color'])) {
				gupfw_error_log($_POST['gupfw_tgfp_in_cart_color'], 0);
				gupfw_optionUPD('gupfw_tgfp_in_cart_color', $_POST['gupfw_tgfp_in_cart_color']);
			}
			if (isset($_POST['gupfw_tgfp_in_cart_fsize'])) {
				gupfw_optionUPD('gupfw_tgfp_in_cart_fsize', sanitize_text_field($_POST['gupfw_tgfp_in_cart_fsize']));
			}
			print '<div class="updated notice notice-success is-dismissible"><p>'. __('Updated', 'gupfw'). '.</p></div>';
		}
	}

	if (isset($_REQUEST['gupfw_submit_action_settings12'])) {
		if (!empty($_POST) && check_admin_referer('gupfw_nonce_action_settings12', 'gupfw_nonce_field_settings12')) {
			if (isset($_POST['gupfw_tgfp_remove_gift_in_cart'])) {
				gupfw_optionUPD('gupfw_tgfp_remove_gift_in_cart', sanitize_text_field($_POST['gupfw_tgfp_remove_gift_in_cart']));
			}
			if (isset($_POST['gupfw_tgfp_accept_gift_in_cart'])) {
				gupfw_optionUPD('gupfw_tgfp_accept_gift_in_cart', sanitize_text_field($_POST['gupfw_tgfp_accept_gift_in_cart']));
			}
			print '<div class="updated notice notice-success is-dismissible"><p>'. __('Updated', 'gupfw'). '.</p></div>';
		}
	}

	if (isset($_REQUEST['gupfw_submit_action_settings2'])) {
		if (!empty($_POST) && check_admin_referer('gupfw_nonce_action_settings2', 'gupfw_nonce_field_settings2')) {
			if (isset($_POST['gupfw_tgfp_in_product_status'])) {
				gupfw_optionUPD('gupfw_tgfp_in_product_status', sanitize_text_field($_POST['gupfw_tgfp_in_product_status']));
			}
			if (isset($_POST['gupfw_tgfp_in_product_content'])) {
				gupfw_optionUPD('gupfw_tgfp_in_product_content', sanitize_text_field($_POST['gupfw_tgfp_in_product_content']));
			}
			if (isset($_POST['gupfw_tgfp_in_product_color'])) {
				gupfw_error_log($_POST['gupfw_tgfp_in_product_color'], 0);
				gupfw_optionUPD('gupfw_tgfp_in_product_color', $_POST['gupfw_tgfp_in_product_color']);
			}
			if (isset($_POST['gupfw_tgfp_in_product_fsize'])) {
				gupfw_optionUPD('gupfw_tgfp_in_product_fsize', sanitize_text_field($_POST['gupfw_tgfp_in_product_fsize']));
			}
			print '<div class="updated notice notice-success is-dismissible"><p>'. __('Updated', 'gupfw'). '.</p></div>';
		}
	}

	if (isset($_REQUEST['gupfw_submit_action_settings3'])) {
		if (!empty($_POST) && check_admin_referer('gupfw_nonce_action_settings3', 'gupfw_nonce_field_settings3')) {
			if (isset($_POST['gupfw_gift_for_any_product_arr'])) {
				gupfw_optionUPD('gupfw_gift_for_any_product_arr', $_POST['gupfw_gift_for_any_product_arr']); 
			} else {
				gupfw_optionUPD('gupfw_gift_for_any_product_arr', array());
			}
			if (isset($_POST['gupfw_gift_for_any_product_in_cart_content'])) {
				gupfw_optionUPD('gupfw_gift_for_any_product_in_cart_content', sanitize_text_field($_POST['gupfw_gift_for_any_product_in_cart_content']));
			}
			if (isset($_POST['gupfw_cart_total_price'])) {
				gupfw_optionUPD('gupfw_cart_total_price', sanitize_text_field($_POST['gupfw_cart_total_price'])); 
			}
			if (isset($_POST['gupfw_whose_price_exceeds'])) {
				gupfw_optionUPD('gupfw_whose_price_exceeds', $_POST['gupfw_whose_price_exceeds']); 
			}
			if (isset($_POST['gupfw_days_of_the_week'])) {
				gupfw_optionUPD('gupfw_days_of_the_week', $_POST['gupfw_days_of_the_week']); 
			} else {
				gupfw_optionUPD('gupfw_days_of_the_week', array());
			}
			if (isset($_POST['gupfw_days_of_the_hours'])) {
				gupfw_optionUPD('gupfw_days_of_the_hours', $_POST['gupfw_days_of_the_hours']); 
			} else {
				gupfw_optionUPD('gupfw_days_of_the_hours', array());
			}		
			print '<div class="updated notice notice-success is-dismissible"><p>'. __('Updated', 'gupfw'). '.</p></div>';
		}
	}	

	/* Сохранение настроек на debug.page */
	if (isset($_REQUEST['gupfw_submit_debug_page'])) {
		if (!empty($_POST) && check_admin_referer('gupfw_nonce_action_debug', 'gupfw_nonce_field_debug')) {
			if (isset($_POST['gupfw_keeplogs'])) {
				gupfw_optionUPD('gupfw_keeplogs', sanitize_text_field($_POST['gupfw_keeplogs']));
				gupfw_error_log('NOTICE: Логи успешно включены; Файл: debug.php; Строка: '.__LINE__, 0);
			} else {
				gupfw_error_log('NOTICE: Логи отключены; Файл: debug.php; Строка: '.__LINE__, 0);
				gupfw_optionUPD('gupfw_keeplogs', '0');
			}
			if (isset($_POST['gupfw_disable_notices'])) {
				gupfw_optionUPD('gupfw_disable_notices', sanitize_text_field($_POST['gupfw_disable_notices']));
			} else {
				gupfw_optionUPD('gupfw_disable_notices', '0');
			}
			print '<div class="updated notice notice-success is-dismissible"><p>'. __('Updated', 'gupfw'). '.</p></div>';
		}
	}
	/* end Сохранение настроек на debug.page */
	/* Очистка логов */
	if (isset($_REQUEST['gupfw_submit_clear_logs'])) {
		$upload_dir = (object)wp_get_upload_dir();
		$name_dir = $upload_dir->basedir."/gift-upon-purchase-for-woocommerce";
		$filename = $name_dir.'/gupfw.log';
		$res = unlink($filename);
		if ($res == true) {
			print '<div class="notice notice-success is-dismissible"><p>' .__('Logs were cleared', 'gupfw'). '.</p></div>';					
		} else {
			print '<div class="notice notice-warning is-dismissible"><p>' .__('Error accessing log file. The log file may have been deleted previously', 'gupfw'). '.</p></div>';		
		}
	}
	/* end Очистка логов */
	/* сброс настроек */
	if (isset($_REQUEST['gupfw_submit_reset'])) {
		if (!empty($_POST) && check_admin_referer('gupfw_nonce_action_reset', 'gupfw_nonce_field_reset')) {
			gupfw_error_log('WARNING: Стартовал сброс настроек плагина; Файл: gift-upon-purchase-for-woocommerce.php; Строка: '.__LINE__, 0);
			$this->on_uninstall();
			$this->on_activation();	
			print '<div class="updated notice notice-success is-dismissible"><p>'. __('The settings have been reset', 'gupfw'). '.</p></div>';
		}
	} 
	/* end сброс настроек */
	/* отправка отчёта */
	if (isset($_REQUEST['gupfw_submit_send_stat'])) {
		if (!empty($_POST) && check_admin_referer('gupfw_nonce_action_send_stat', 'gupfw_nonce_field_send_stat')) { 	
		if (is_multisite()) { 
			$gupfw_is_multisite = 'включен';	
			$gupfw_keeplogs = get_blog_option(get_current_blog_id(), 'gupfw_keeplogs');
		} else {
			$gupfw_is_multisite = 'отключен'; 
			$gupfw_keeplogs = get_option('gupfw_keeplogs');
		}
		$mail_content = "Версия плагина: ". gupfw_VER . PHP_EOL;
		$mail_content .= "Версия WP: ".get_bloginfo('version'). PHP_EOL;	 
		$woo_version = gupfw_get_woo_version_number();
		$mail_content .= "Версия WC: ".$woo_version. PHP_EOL;
		$mail_content .= "Адрес сайта: ".get_site_url(). PHP_EOL;	   
		$mail_content .= "Режим мультисайта: ".$gupfw_is_multisite. PHP_EOL;
		$mail_content .= "Вести логи: ".$gupfw_keeplogs. PHP_EOL;
		$mail_content .= "Расположение логов: ". gupfw_UPLOAD_DIR .'/gupfw.log'. PHP_EOL;
		$possible_problems_arr = gupfw_possible_problems_list();
		if ($possible_problems_arr[1] > 0) {
			$possible_problems_arr[3] = str_replace('<br/>', PHP_EOL, $possible_problems_arr[3]);
			$mail_content .= "Самодиагностика: ". PHP_EOL .$possible_problems_arr[3];
		} else {
			$mail_content .= "Самодиагностика: Функции самодиагностики не выявили потенциальных проблем". PHP_EOL;
		}	  
		if (isset($_REQUEST['gupfw_its_ok'])) {
			$mail_content .= PHP_EOL ."Помог ли плагин: ".sanitize_text_field($_REQUEST['gupfw_its_ok']);
		}
		if (isset($_POST['gupfw_email'])) {
			$mail_content .= PHP_EOL ."Почта: ".sanitize_text_field($_POST['gupfw_email']);
		}
		if (isset($_POST['gupfw_message'])) {
			$mail_content .= PHP_EOL ."Сообщение: ".sanitize_text_field($_POST['gupfw_message']);
		}
		$argsp = array('post_type' => 'product', 'post_status' => 'publish', 'posts_per_page' => -1 );
		$products = new WP_Query($argsp);
		$vsegotovarov = $products->found_posts;
		$mail_content .= PHP_EOL ."Число товаров на выгрузку: ". $vsegotovarov. PHP_EOL;
		$gupfw_errors = gupfw_optionGET('gupfw_errors');
		$mail_content .= "Ошибки: ".$gupfw_errors;
		wp_mail('support@icopydoc.ru', 'Отчёт Gift upon purchase for WooCommerce', $mail_content);
		print '<div class="updated notice notice-success is-dismissible"><p>'. __('The data has been sent. Thank you', 'gupfw'). '.</p></div>';
		}		
	}
 } 

 public static function gupfw_plugin_action_links($actions, $plugin_file) {
	if (false === strpos($plugin_file, basename(__FILE__))) {
		// проверка, что у нас текущий плагин
		return $actions;
	}
	$settings_link = '<a href="/wp-admin/admin.php?page=gupfwsettings">'. __('Settings', 'gupfw').'</a>';
	array_unshift($actions, $settings_link); 
	return $actions; 
 }

 public static function gupfw_added_wc_tabs($tabs) {
	$tabs['gupfw_special_panel'] = array(
		'label' => __('Add Gift', 'gupfw'), // название вкладки
		'target' => 'gupfw_added_wc_tabs', // идентификатор вкладки
		'class' => array('hide_if_grouped'), // классы управления видимостью вкладки в зависимости от типа товара
		'priority' => 71, // приоритет вывода
	);
	return $tabs;
 }

 public static function gupfw_art_added_tabs_icon() { ?>
	<style>
		#woocommerce-coupon-data ul.wc-tabs li.gupfw_special_panel_options a::before,
		#woocommerce-product-data ul.wc-tabs li.gupfw_special_panel_options a::before,
		.woocommerce ul.wc-tabs li.gupfw_special_panel_options a::before {		
			content: "\f456";
		}
	</style>
	<?php
 }

 public static function gupfw_art_added_tabs_panel() {
	global $post;
	?>
	<div id="gupfw_added_wc_tabs" class="panel woocommerce_options_panel">
		<?php do_action('gupfw_before_options_group', $post); ?>
		<div class="options_group">
			<h2><strong><?php _e('Here you can customize the gifts that the user will receive', 'gupfw'); ?></strong></h2>
			<?php do_action('gupfw_prepend_options_group', $post); ?>
			<?php // Чекбокс
				woocommerce_wp_checkbox(array(
					'id'            => '_gupfw_stopgift',
				//	'wrapper_class' => 'show_if_simple',
					'label'         => __('Disable gifts', 'gupfw'),
					'description'   => __('Disable gifts for this product', 'gupfw'),
				));
				woocommerce_wp_checkbox(array(
					'id'            => '_two_price_one',
					'wrapper_class' => 'show_if_simple', // не показываем если товар вариативный
					'label'         => __('Two for the price of one', 'gupfw'),
					'description'   => '<a href="https://icopydoc.ru/kak-dobavit-dva-tovara-po-tsene-odnogo-v-woocommerce/?utm_source=gift-upon-purchase-for-woocommerce&utm_medium=organic&utm_campaign=in-plugin-gift-upon-purchase-for-woocommerce&utm_content=edit_product&utm_term=two-price-one-instruction" target="_blank">'. __('How it works', 'gupfw').'</a>',
				));
				gupfw_woocommerce_wp_select_multiple(array(
					'id' => '_days_of_the_week',
				//	'wrapper_class' => 'show_if_simple', 
					'label' => __('Gift only on certain days', 'gupfw'),
					'options' => array(
						'Monday' => __('Monday', 'gupfw'),
						'Tuesday' => __('Tuesday', 'gupfw'),
						'Wednesday' => __('Wednesday', 'gupfw'),
						'Thursday' => __('Thursday', 'gupfw'),
						'Friday' => __('Friday', 'gupfw'),
						'Saturday' => __('Saturday', 'gupfw'),
						'Sunday' => __('Sunday', 'gupfw'),
					)
				));
				$res_arr = array(); for ($i = 1; $i<25; $i++) {$res_arr[$i] = $i;}
				gupfw_woocommerce_wp_select_multiple(array(
					'id' => '_days_of_the_hours',
				//	'wrapper_class' => 'show_if_simple',
					'label' => __('Gift only at certain hours', 'gupfw'),
					'options' => $res_arr
				));
				?>			
			<p class="form-field product_field_type">
			<label for="product_field_type"><?php _e('Product selection', 'gupfw'); ?></label> <select id="product_field_type" name="product_field_type[]" class="wc-product-search" multiple="multiple" style="width: 50%;" data-placeholder="<?php esc_attr_e('Search for a product…', 'gupfw'); ?>" data-action="woocommerce_json_search_products_and_variations" data-exclude="<?php echo intval($post->ID); ?>">
				<?php /* 
				<select id="product_field_type" name="product_field_type[]" class="wc-product-search" multiple="multiple" style="width: 50%;" data-placeholder="<?php esc_attr_e('Search for a product…', 'gupfw'); ?>" data-action="woocommerce_json_search_products_and_variations" data-exclude="<?php echo intval($post->ID); ?>"> 
				data-exclude="<?php echo intval($post->ID); ?>" - конструкция исключает из поиска товары с определённым id (подарок за самого себя)
				*/
				$gupfw_product_gift_ids = get_post_meta($post->ID, '_gupfw_product_gift_ids', true);
				$product_ids = ! empty($gupfw_product_gift_ids) && isset($gupfw_product_gift_ids) ? array_map('absint',  $gupfw_product_gift_ids) : array();
				if ($product_ids) {
					foreach ($product_ids as $product_id) {
						$product = wc_get_product($product_id);
						$product_name = $product->get_formatted_name();
						echo '<option value="'.esc_attr($product_id).'" '.selected(true, true, false).'>'.esc_html($product->get_formatted_name()).'</option>';
					}
				}
				?>
			</select><span class="woocommerce-help-tip" data-tip="<?php _e('Select the products that the user will receive as a gift', 'gupfw'); ?>"></span>
			</p>
			<?php do_action('gupfw_append_options_group', $post); ?>
		</div>
		<?php do_action('gupfw_after_options_group', $post); ?>
	</div>
	<?php
 } 

 public static function gupfw_art_woo_custom_fields_save($post_id) {
	// Сохранение произвольного поля по выбору товаров с поиском
 	if (isset( $_POST['product_field_type'] ) && !empty($_POST['product_field_type'] )) {
		// Проверяем данные, если они существуют и не пустые, то записываем данные в поле
		update_post_meta($post_id, '_gupfw_product_gift_ids',  array_map('absint', (array) $_POST['product_field_type']));
	} else {
		// Иначе удаляем созданное поле из бд
		delete_post_meta($post_id, '_gupfw_product_gift_ids');
	}
	if (isset($_POST['_gupfw_stopgift']) && !empty($_POST['_gupfw_stopgift'] )) {
		// Проверяем данные, если они существуют и не пустые, то записываем данные в поле
		update_post_meta($post_id, '_gupfw_stopgift', sanitize_text_field($_POST['_gupfw_stopgift']));
	} else {
		// Иначе удаляем созданное поле из бд
		update_post_meta($post_id, '_gupfw_stopgift', '');
	}
	if (isset($_POST['_days_of_the_week']) && !empty($_POST['_days_of_the_week'] )) {
		update_post_meta($post_id, '_days_of_the_week', $_POST['_days_of_the_week']);
	} else {
		update_post_meta($post_id, '_days_of_the_week', array());
	}
	if (isset($_POST['_days_of_the_hours']) && !empty($_POST['_days_of_the_hours'] )) {
		update_post_meta($post_id, '_days_of_the_hours', $_POST['_days_of_the_hours']);
	} else {
		update_post_meta($post_id, '_days_of_the_hours', array());
	}
	if (isset($_POST['_two_price_one']) && !empty($_POST['_two_price_one'] )) {
		// Проверяем данные, если они существуют и не пустые, то записываем данные в поле
		update_post_meta($post_id, '_two_price_one', sanitize_text_field($_POST['_two_price_one']));
	} else {
		// Иначе удаляем созданное поле из бд
		update_post_meta($post_id, '_two_price_one', '');
	}		
 }

 public static function gupfw_get_posts_ajax_callback() {
	// we will pass post IDs and titles to this array
	$return = array();
 
	// you can use WP_Query, query_posts() or get_posts() here - it doesn't matter
	$search_results = new WP_Query( array( 
		's'=> $_GET['q'], // the search query
		'post_status' => 'publish', // if you don't want drafts to be returned
		'post_type' => array('product', 'product_variation'), 
		'ignore_sticky_posts' => 1,
		'posts_per_page' => 50 // how much to show at once
	) );
	if ($search_results->have_posts()) :
		while($search_results->have_posts()) : $search_results->the_post();	
			// shorten the title a little
			$title = (mb_strlen($search_results->post->post_title) > 50) ? mb_substr($search_results->post->post_title, 0, 49) . '...' : $search_results->post->post_title;
			$return[] = array($search_results->post->ID, $title.' ('.$search_results->post->post_name.')'); // array( Post ID, Post Title )
		endwhile;
	endif;
	echo json_encode($return);
	die;
 }  

 // текст перед метаданными в карточке товара
 public static function gupfw_art_get_text_field_after_add_card() {
	$gupfw_tgfp_in_product_status = gupfw_optionGET('gupfw_tgfp_in_product_status');
	if ($gupfw_tgfp_in_product_status === 'hide') {return;}
	$gupfw_tgfp_in_product_content = stripslashes(htmlspecialchars(gupfw_optionGET('gupfw_tgfp_in_product_content'))); 
	$gupfw_tgfp_in_product_color = gupfw_optionGET('gupfw_tgfp_in_product_color');
	$gupfw_tgfp_in_product_fsize = gupfw_optionGET('gupfw_tgfp_in_product_fsize');	 

	global $post;
	$gupfw_stopgift = get_post_meta($post->ID, '_gupfw_stopgift', true);
	if ($gupfw_stopgift === 'yes') {return;}

	if (get_post_meta($post->ID, '_days_of_the_week', true) !== '') {
		$curDate = current_time('l');
		$days_of_the_week = get_post_meta($post->ID, '_days_of_the_week', true);
		if (is_array($days_of_the_week) && !empty($days_of_the_week)) {
			if (!in_array($curTime, $days_of_the_hours)) {return;}
		}
	}
	if (get_post_meta($post->ID, '_days_of_the_hours', true) !== '') {
		$curTime = current_time('H');
		$days_of_the_hours = get_post_meta($post->ID, '_days_of_the_hours', true);
		if (is_array($days_of_the_hours) && !empty($days_of_the_hours)) {
			if (!in_array($curTime, $days_of_the_hours)) {return;}
		}
	}

	$gupfw_product_gift_ids_arr = get_post_meta($post->ID, '_gupfw_product_gift_ids', true);
	if (!empty($gupfw_product_gift_ids_arr)) {
		for ($i = 0; $i < count($gupfw_product_gift_ids_arr); $i++) {
			$gift_id = $gupfw_product_gift_ids_arr[$i];
			$gift_product = wc_get_product($gift_id);
			if ($gift_product == null) {continue;}
			echo '<span style="color: '.$gupfw_tgfp_in_product_color.'; font-size: '.$gupfw_tgfp_in_product_fsize.'px;">'.$gupfw_tgfp_in_product_content.'</span>: <a href="'.get_permalink($gift_product->get_id()).'" class="gupfw_link_to_gift" style="font-size: '.$gupfw_tgfp_in_product_fsize.'px;">'.$gift_product->get_title().'</a><br />';
		}
	}

	// надпись Два товара по цене одного
	if (get_post_meta($post->ID, '_two_price_one', true) === 'yes') {
		echo '<span style="color: '.$gupfw_tgfp_in_product_color.'; font-size: '.$gupfw_tgfp_in_product_fsize.'px;">'. __('Two products for the price of one', 'gupfw').'</span><br />';
	} 	
 }  

 // ниже этой линии нужны правки

 // функция добавляет подарок в корзину
 public static function gupfw_add_product_to_cart() {
	gupfw_error_log('Стартовала gupfw_add_product_to_cart; Файл: gift-upon-purchase-for-woocommerce.php; Строка: '.__LINE__, 0);	 
	if (!is_admin()) {
		$gift_ids_arr = array(); // id полагающихся пользователю подарокв
		// проверяем все товары в корзине на полагающиеся подарки	
		$products_in_cart_arr = WC()->cart->get_cart(); // товары в корзине
		$cart_totals_arr = WC()->cart->get_totals();
		$cart_total = $cart_totals_arr['total'];
			
		gupfw_error_log('gupfw_add_product_to_cart вызывает gupfw_gift_check; Файл: gift-upon-purchase-for-woocommerce.php; Строка: '.__LINE__, 0);
		if (gupfw_gift_check($products_in_cart_arr, $cart_total)) {
			$gupfw_gift_for_any_product_arr = gupfw_optionGET('gupfw_gift_for_any_product_arr');			
			$gift_ids_arr = array_merge($gift_ids_arr, $gupfw_gift_for_any_product_arr);
		}

		if (sizeof($products_in_cart_arr) > 0) {
			foreach ($products_in_cart_arr as $cart_item_key => $values) {
				if ($values['variation_id'] == 0) {
					$_product = $values['data'];
					$product_id_in_cart = $_product->get_id();
				} else {
					$product_id_in_cart = $values['product_id'];
					gupfw_error_log('В корзине лежит вариация. id товара = '.$product_id_in_cart.', id вариации = '.$values['variation_id'].'; Файл: gift-upon-purchase-for-woocommerce.php; Строка: '.__LINE__, 0);
				}
				if (get_post_meta($product_id_in_cart, '_gupfw_product_gift_ids', true) !== '') {
					$gupfw_product_gift_ids_arr = get_post_meta($product_id_in_cart, '_gupfw_product_gift_ids', true);
					if (!empty($gupfw_product_gift_ids_arr)) { 

						$curDate = current_time('l');
						$curTime = current_time('H');

						for ($i = 0; $i < count($gupfw_product_gift_ids_arr); $i++) {

							if (get_post_meta($product_id_in_cart, '_days_of_the_week', true) !== '') {
								$days_of_the_week = get_post_meta($product_id_in_cart, '_days_of_the_week', true);
								if (is_array($days_of_the_week) && !empty($days_of_the_week)) {
									if (!in_array($curDate, $days_of_the_week)) {continue;}
								}
							}
							if (get_post_meta($product_id_in_cart, '_days_of_the_hours', true) !== '') {
								$days_of_the_hours = get_post_meta($product_id_in_cart, '_days_of_the_hours', true);
								if (is_array($days_of_the_hours) && !empty($days_of_the_hours)) {
									if (!in_array($curTime, $days_of_the_hours)) {continue;}
								}
							}

							$gift_ids_arr[] = $gupfw_product_gift_ids_arr[$i];
						}
					}
				}

				// добавляем в корзину подарок за самого себя
				if (get_post_meta($product_id_in_cart, '_two_price_one', true) === 'yes') {
					// $cart_item_quantity = $values['quantity'] + 1;
					if ($values['quantity'] == 1) {
						WC()->cart->set_quantity($cart_item_key, 2);
					}
				}	
			}
		}
 
		// добавим товары-подарки в корзину, если они полагаются
		if (!empty($gift_ids_arr)) {
			for ($i = 0; $i < count($gift_ids_arr); $i++ ) {
				$product_id_for_gift = $gift_ids_arr[$i]; 
				$found = false;
				//check if product already in cart
				if (sizeof($products_in_cart_arr) > 0) {
					foreach ($products_in_cart_arr as $cart_item_key => $values ) {
						$_product = $values['data'];
						if ($_product->get_id() == $product_id_for_gift) {
							$found = true;
							$cart_item_key_for_del = $cart_item_key;
						}
					}
					// if product not found, add it
					if ($found !== true && !isset($_COOKIE["gupfw_no_gift"])) {
						WC()->cart->add_to_cart($product_id_for_gift);
					} else if ($found === true) {
						if (isset($_REQUEST['gupfw_refuse_gifts'])) {
							WC()->cart->remove_cart_item($cart_item_key_for_del);
						}
					}
				}
			}
		}
	}
 }

 public static function gupfw_change_recalc_price($cart) {
	if (is_admin() && ! defined('DOING_AJAX')) {return;}
	// you can always print_r() your object and look what's inside
	// print_r($cart); exit;
	$gift_ids_arr = array(); // id полагающихся пользователю подарокв
	$products_in_cart_arr = $cart->get_cart(); // массив всех товаров в корзине
	if (empty($products_in_cart_arr)) {return;}

	$curDate = current_time('l');
	$curTime = current_time('H');

	foreach($products_in_cart_arr as $cart_item) {
		$product_id_in_cart = $cart_item["product_id"];
		$gupfw_stopgift = get_post_meta($product_id_in_cart, '_gupfw_stopgift', true);
		if ($gupfw_stopgift === 'yes') {continue;}

		if (get_post_meta($product_id_in_cart, '_days_of_the_week', true) !== '') {
			$days_of_the_week = get_post_meta($product_id_in_cart, '_days_of_the_week', true);
			if (is_array($days_of_the_week) && !empty($days_of_the_week)) {
				if (!in_array($curDate, $days_of_the_week)) {continue;}
			}
		}
		if (get_post_meta($product_id_in_cart, '_days_of_the_hours', true) !== '') {		
			$days_of_the_hours = get_post_meta($product_id_in_cart, '_days_of_the_hours', true);
			if (is_array($days_of_the_hours) && !empty($days_of_the_hours)) {
				if (!in_array($curTime, $days_of_the_hours)) {continue;}
			}
		}

		if (get_post_meta($product_id_in_cart, '_gupfw_product_gift_ids', true) !== '') {
			$gupfw_product_gift_ids_arr = get_post_meta($product_id_in_cart, '_gupfw_product_gift_ids', true);
			if (!empty($gupfw_product_gift_ids_arr)) {
				for ($i = 0; $i < count($gupfw_product_gift_ids_arr); $i++) {
					$gift_ids_arr[] = $gupfw_product_gift_ids_arr[$i];
				}
			}
		}

		// уменьшим стоимость товара
		if (get_post_meta($cart_item["product_id"], '_two_price_one', true) === 'yes') {
			if ($cart_item['quantity'] > 1) {
				// https://www.pandoge.com/haki/skidka-v-procentah-na-vsyu-korzinu-bez-kupona-v-woocommerce
				// https://misha.agency/woocommerce/dobavlenie-kuponov-programmno.html 
				$discount = $cart_item['data']->get_price();
				$cart->add_fee('100% '. __('discount for one item', 'gupfw').' "'.$cart_item['data']->get_name().'"', -$discount);
			}
		}  	
	}
	foreach ($products_in_cart_arr as $hash => $cart_item ) {
		gupfw_error_log($cart_item, 0);		
		if (in_array($cart_item["product_id"], $gift_ids_arr) || in_array($cart_item["variation_id"], $gift_ids_arr)) { // Если в корзине есть товары с ID = подарка
			if ($cart_item['quantity'] < 2) {
				if (!isset($_COOKIE["gupfw_no_gift"])) { // нет отказа от подарков
					$cart_item['data']->set_price(0);
				}
			} 
		}
	}
	
	$cart_totals_arr = $cart->get_totals();
	$cart_total = $cart_totals_arr['total'];
	gupfw_error_log('gupfw_change_recalc_price вызывает gupfw_gift_check; Файл: gift-upon-purchase-for-woocommerce.php; Строка: '.__LINE__, 0);
	if (gupfw_gift_check($products_in_cart_arr, $cart_total, $gift_ids_arr) === true) {
		$gupfw_gift_for_any_product_arr = gupfw_optionGET('gupfw_gift_for_any_product_arr');
		$gift_ids_arr = array_merge($gift_ids_arr, $gupfw_gift_for_any_product_arr);
	} 

	foreach ($products_in_cart_arr as $hash => $cart_item) {
		gupfw_error_log($cart_item, 0);		
		if (in_array($cart_item["product_id"], $gift_ids_arr) || in_array($cart_item["variation_id"], $gift_ids_arr)) { // Если в корзине есть товары с ID = подарка
			if ($cart_item['quantity'] < 2) {
				if (!isset($_COOKIE["gupfw_no_gift"])) { // нет отказа от подарков
					$cart_item['data']->set_price(0);
				}
			} 
		}
	}	
 } 

 function gupfw_add_excerpt_in_cart_item_name($item_name, $cart_item, $cart_item_key) {
	$gupfw_tgfp_in_cart_status = gupfw_optionGET('gupfw_tgfp_in_cart_status');
	if ($gupfw_tgfp_in_cart_status === 'hide') {return $item_name;}
	$gupfw_tgfp_in_cart_content = stripslashes(htmlspecialchars(gupfw_optionGET('gupfw_tgfp_in_cart_content'))); 
	$gupfw_gift_for_any_product_in_cart_content = stripslashes(htmlspecialchars(gupfw_optionGET('gupfw_gift_for_any_product_in_cart_content'))); 
	$gupfw_tgfp_in_cart_color = gupfw_optionGET('gupfw_tgfp_in_cart_color');
	$gupfw_tgfp_in_cart_fsize = gupfw_optionGET('gupfw_tgfp_in_cart_fsize');

	global $woocommerce;
	$products_in_cart_arr = $woocommerce->cart->get_cart();
	$gift_ids_arr = array(); // массив id подарокв

	$curDate = current_time('l');
	$curTime = current_time('H');
	$additionaltext = '';
	foreach($products_in_cart_arr as $cur_cart_item) {
		$product_id_in_cart = $cur_cart_item["product_id"];
		$gupfw_stopgift = get_post_meta($product_id_in_cart, '_gupfw_stopgift', true);
		if ($gupfw_stopgift === 'yes') {continue;}	
		
		if (get_post_meta($product_id_in_cart, '_days_of_the_week', true) !== '') {
			$days_of_the_week = get_post_meta($product_id_in_cart, '_days_of_the_week', true);
			if (is_array($days_of_the_week) && !empty($days_of_the_week)) {
				if (!in_array($curDate, $days_of_the_week)) {continue;}
			}
		}
		if (get_post_meta($product_id_in_cart, '_days_of_the_hours', true) !== '') {
			$days_of_the_hours = get_post_meta($product_id_in_cart, '_days_of_the_hours', true);
			if (is_array($days_of_the_hours) && !empty($days_of_the_hours)) {
				if (!in_array($curTime, $days_of_the_hours)) {continue;}
			}
		}

		// текст Один товар бесплатно
		if (get_post_meta($product_id_in_cart, '_two_price_one', true) === 'yes') {
			$additionaltext = '<br /><span style="font-size: '.$gupfw_tgfp_in_cart_fsize.'px; color: '.$gupfw_tgfp_in_cart_color.';">'. __('One item for free. See details below', 'gupfw'). '</span>'; 
		} 

		if (get_post_meta($product_id_in_cart, '_gupfw_product_gift_ids', true) !== '') {
			$gupfw_product_gift_ids_arr = get_post_meta($product_id_in_cart, '_gupfw_product_gift_ids', true);
			if (!empty($gupfw_product_gift_ids_arr)) {
				for ($i = 0; $i < count($gupfw_product_gift_ids_arr); $i++) {
					$gift_ids_arr[] = $gupfw_product_gift_ids_arr[$i];
				}
			}
		}
	}
	
	if (in_array($cart_item["product_id"], $gift_ids_arr) || in_array($cart_item["variation_id"], $gift_ids_arr)) { // Если в корзине есть товары с ID = подарку
		if ($cart_item['quantity'] < 2) {
			if (!isset($_COOKIE["gupfw_no_gift"])) {
				$additionaltext = '<br /><span style="font-size: '.$gupfw_tgfp_in_cart_fsize.'px; color: '.$gupfw_tgfp_in_cart_color.';">'.$gupfw_tgfp_in_cart_content.' '.$cart_item['data']->name.'</span>'; 
			}
		}
	}

	$cart_totals_arr = $woocommerce->cart->get_totals();
	$cart_total = $cart_totals_arr['total'];
		
	gupfw_error_log('gupfw_add_excerpt_in_cart_item_name вызывает gupfw_gift_check; Файл: gift-upon-purchase-for-woocommerce.php; Строка: '.__LINE__, 0);
	if (gupfw_gift_check($products_in_cart_arr, $cart_total)) {
		$gupfw_gift_for_any_product_arr = gupfw_optionGET('gupfw_gift_for_any_product_arr');
		if (in_array($cart_item["product_id"], $gupfw_gift_for_any_product_arr) || in_array($cart_item["variation_id"], $gupfw_gift_for_any_product_arr)) { // Если в корзине есть товары с ID = подарку			
			if ($cart_item['quantity'] < 2) {
				$gupfw_cart_total_price = gupfw_optionGET('gupfw_cart_total_price');
				$cart_total = gupfw_cart_total_without_gifts($products_in_cart_arr, $gupfw_gift_for_any_product_arr, $gift_ids_arr);
				if ($cart_total > $gupfw_cart_total_price) {
					if (!isset($_COOKIE["gupfw_no_gift"])) {
						$additionaltext = '<br /><span style="font-size: '.$gupfw_tgfp_in_cart_fsize.'px; color: '.$gupfw_tgfp_in_cart_color.';">'.$gupfw_gift_for_any_product_in_cart_content.'</span>'; 
					}
				}
			}
		}
	}	

	return $item_name . $additionaltext;	
 } 

 // установка куки отказа от подарков
 public static function gupfw_set_new_cookie() { // setting your cookies there
	if (isset($_REQUEST['gupfw_refuse_gifts'])) {
		if (!isset($_COOKIE["gupfw_no_gift"])) {	
			$value = 'on';	 
			$expires_time = (int)current_time('timestamp') + 3600; // установить cookie на 1 час
			setcookie('gupfw_no_gift', $value, $expires_time, '/');	// устанавливаем куку на весь сайт (слэш)
			$_COOKIE["gupfw_no_gift"] = 'on'; // пришлось добавить чтобы кнопка "Принять/убрать подарки" работала синхронно
		}
	}
	if (isset($_REQUEST['gupfw_accept_gifts'])) {
		$expires_time = (int)current_time('timestamp') - 3600; // установить cookie на -1 час
		setcookie('gupfw_no_gift', '', $expires_time, '/'); // удаляем куку со всего сайта
		unset($_COOKIE["gupfw_no_gift"]);
	}
 }

 // кнопка отказа от подарков
 public static function gupfw_before_add_to_cart_btn() { 
	if (isset($_COOKIE["gupfw_no_gift"])) {
		$v = __('Accept gifts', 'gupfw');
		$name = 'gupfw_accept_gifts';
	} else {
		$v = __('Remove gifts', 'gupfw');
		$name = 'gupfw_refuse_gifts';
	}
	echo '<button type="submit" class="button" name="'.$name.'">'.$v.'</button>';
 } 
} /* end class GiftUponPurchaseForWooCommerce */
?>